package dev.shadowsoffire.apotheosis.adventure.affix.salvaging;

import java.util.Iterator;
import it.unimi.dsi.fastutil.ints.IntSet;
import java.util.List;
import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.items.IItemHandler;
import net.minecraft.world.level.Level;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraftforge.common.capabilities.ForgeCapabilities;
import net.minecraft.core.Direction;
import net.minecraftforge.common.capabilities.Capability;
import dev.shadowsoffire.apotheosis.Apoth;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.BlockPos;
import net.minecraftforge.common.util.LazyOptional;
import dev.shadowsoffire.placebo.cap.InternalItemHandler;
import net.minecraft.world.level.block.entity.BlockEntity;

public class SalvagingTableTile extends BlockEntity
{
    protected final InternalItemHandler output;
    protected final LazyOptional<SalvagingItemHandler> itemHandler;
    
    public SalvagingTableTile(final BlockPos pPos, final BlockState pBlockState) {
        super((BlockEntityType)Apoth.Tiles.SALVAGING_TABLE.get(), pPos, pBlockState);
        this.output = new InternalItemHandler(6);
        this.itemHandler = (LazyOptional<SalvagingItemHandler>)LazyOptional.of(() -> new SalvagingItemHandler());
    }
    
    public <T> LazyOptional<T> getCapability(final Capability<T> cap, final Direction side) {
        if (cap == ForgeCapabilities.ITEM_HANDLER) {
            return (LazyOptional<T>)this.itemHandler.cast();
        }
        return (LazyOptional<T>)super.getCapability((Capability)cap, side);
    }
    
    protected void m_183515_(final CompoundTag tag) {
        tag.m_128365_("output", (Tag)this.output.serializeNBT());
        super.m_183515_(tag);
    }
    
    public void m_142466_(final CompoundTag tag) {
        if (tag.m_128441_("output")) {
            this.output.deserializeNBT(tag.m_128469_("output"));
        }
        super.m_142466_(tag);
    }
    
    protected class SalvagingItemHandler implements IItemHandler
    {
        public int getSlots() {
            return 1 + SalvagingTableTile.this.output.getSlots();
        }
        
        public ItemStack getStackInSlot(final int slot) {
            if (slot == 0) {
                return ItemStack.f_41583_;
            }
            return SalvagingTableTile.this.output.getStackInSlot(slot - 1);
        }
        
        public ItemStack insertItem(final int slot, final ItemStack stack, final boolean simulate) {
            if (slot != 0) {
                return stack;
            }
            final List<ItemStack> outputs = SalvagingMenu.getBestPossibleSalvageResults(SalvagingTableTile.this.f_58857_, stack);
            if (outputs.isEmpty()) {
                return stack;
            }
            final IntSet skipSlots = (IntSet)new IntOpenHashSet();
            for (ItemStack out : outputs) {
                for (int i = 0; i < 6; ++i) {
                    if (!skipSlots.contains(i)) {
                        final int size = out.m_41613_();
                        out = SalvagingTableTile.this.output.insertItem(i, out, true);
                        if (size != out.m_41613_()) {
                            skipSlots.add(i);
                        }
                        if (out.m_41619_()) {
                            break;
                        }
                    }
                }
                if (!out.m_41619_()) {
                    return stack;
                }
            }
            if (!simulate) {
                for (ItemStack out : outputs) {
                    for (int i = 0; i < 6; ++i) {
                        out = SalvagingTableTile.this.output.insertItem(i, out, false);
                        if (out.m_41619_()) {
                            break;
                        }
                    }
                    if (!out.m_41619_()) {
                        return stack;
                    }
                }
            }
            return ItemStack.f_41583_;
        }
        
        public ItemStack extractItem(final int slot, final int amount, final boolean simulate) {
            if (slot == 0) {
                return ItemStack.f_41583_;
            }
            return SalvagingTableTile.this.output.extractItem(slot - 1, amount, simulate);
        }
        
        public int getSlotLimit(final int slot) {
            if (slot == 0) {
                return 1;
            }
            return SalvagingTableTile.this.output.getSlotLimit(slot - 1);
        }
        
        public boolean isItemValid(final int slot, final ItemStack stack) {
            return slot == 0 && SalvagingMenu.findMatch(SalvagingTableTile.this.f_58857_, stack) != null;
        }
    }
}
